/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : pop.h                                                    */
/*                                                                       */
/*       Date : November  6, 1997                                        */
/* Updated at : November  6, 1997                                        */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#ifndef _POP_
#define _POP_

#include "global.h"
#include "gene.h"
#include "chromo.h"


struct struct_Population
{
   type_Gene **genePool;      /* pointer to original gene pool vector    */
   unsigned int chromoLength; /* length of chromosomes in the population */
   eval_long_t eval;          /* sum of all chromosome evaluation values */
   int popSize;               /* number of chromosomes in the population */
   type_Chromosome **myChromoPool; 

};

typedef struct struct_Population type_Population;


type_Population* definePopulation(int _popSize, unsigned int _chromoLength, 
                                  type_Gene *_genePool[]);

type_Population* copyPopulation(type_Population* RHS);

/* Population initialization. */
void initPopulation(type_Population* this);

/*
// Calculates (if necessary) fitness for each chromosome in the
// population and returns the sum of the fitness values.
*/
eval_long_t fitness(type_Population* this, enum fitness_t fitnessType, 
                    eval_t fitnessParam);

/*
// Creates a new population using this population as a parent pool
// and returns a pointer to the new population.
// If deletionType == deleteAll, the user of this class has to
// delete this population (the new one is really a _new_ one).
// If deletionType == steadyDelete, the old population is altered
// and thus, requires no further action by the user.
*/
type_Population* newPop(type_Population* this,
               enum fitness_t fitnessType, eval_t fitnessParam,
	       float crossover, enum cross_t crossoverType, float mutation,
	       enum delete_t deletionType, int elitismValue);

/* Returns the fitness of the best cromosome in the population.  */
eval_t bestFitness(type_Population* this); 

/* Returns the modified fitness of the best cromosome in the population.  */
eval_t bestFitnessMod(type_Population* this); 

/* Returns the average chromosome fitness of the population. */
eval_t fitnessAverage(type_Population* this);

/* Returns the average modified chromosome fitness of the population. */
eval_t fitnessAverageMod(type_Population* this); 

/* Returns the best chromosome in the population. */
type_Chromosome* bestChromo(type_Population* this); 

/* Replaces the worst Chromosome with the given one  */
void replaceWorstChromosomeWith(type_Population* this, 
                                type_Chromosome* newChromo);
   
void disposePopulation(type_Population* this);

#endif
